#!/bin/bash
set -e

# This script will build the training corpus, train a machine translation
#  baseline model, and evaluate it on the test set. It can be run simply
#  using:
#   nohup ./process.sh &> process.log &
#  You can follow the process of the training by using
#   tail -f process.log
#
# It requires the following tools be installed and accessible
#  For building data -->
#    KyTea:  http://www.phontron.com/kytea/  (ver. 0.2.0 or higher)
#    lader:  http://www.phontron.com/lader/
#  For the MT system -->
#    GIZA++: http://giza-pp.googlecode.com/
#    Moses:  http://www.statmt.org/moses/
#    SRILM:  http://www.speech.sri.com/projects/srilm/download.html
# 
#  MOSES_DIR must be set to the directory of moses
#  MT_SCRIPTS must be set to the directory containing the MT helper scripts
#   and the evaluation script
#
# In order to do this, you can follow the directions of Part I here:
#  http://www.statmt.org/moses_steps.html
# then install KyTea separately.
#
# If you have any problems running this, please report them to
# Graham Neubig at by sending a mail to [his last name] at gmail.com

# How many threads to use
THREADS=16
# Which directory to put the experimental files in
WORK_DIR=results

# 0) first set the path of all variables here
WD=`pwd`
# MT_SCRIPTS=
MOSES_DIR=$HOME/work/mosesdecoder
LADER_DIR=$HOME/work/lader
EXTERNAL_BIN_DIR=$HOME/usr/local/giza-pp
# the path to KyTea
KYTEA=`which kytea` \
    && type -P $KYTEA &>/dev/null \
    || { echo "Could not find KyTea at '$KYTEA'. Aborting." >&2; exit 1; }
# the path to Moses
MOSES=$MOSES_DIR/bin/moses \
    && type -P $MOSES &>/dev/null \
    || { echo "Could not find Moses at '$MOSES'. Aborting." >&2; exit 1; }
# the path to lader
LADER=$LADER_DIR/src/bin/lader \
    && type -P $LADER &>/dev/null \
    || { echo "Could not find lader at '$LADER'. Aborting." >&2; exit 1; }
TRAIN_LADER=$LADER_DIR/src/bin/train-lader \
    && type -P $TRAIN_LADER &>/dev/null \
    || { echo "Could not find train-lader at '$TRAIN_LADER'. Aborting." >&2; exit 1; }
# the path to ngram-count of SRILM
NGRAM_COUNT=`which ngram-count` \
    && type -P $NGRAM_COUNT &>/dev/null \
    || { echo "Could not find SRILM at '$NGRAM_COUNT'. Aborting." >&2; exit 1; }

# 1) perform a sanity check
type -P $MT_SCRIPTS/tokenizer.perl &>/dev/null || { echo "Could not find Tokenizer at $MT_SCRIPTS/tokenizer.perl. Aborting." >&2; exit 1; }
type -P $MOSES_DIR/scripts/training/clean-corpus-n.perl &>/dev/null || { echo "Could not find corpus cleaner at $MOSES_DIR/scripts/training/clean-corpus-n.perl. Aborting." >&2; exit 1; }

# 2) extract the sentences
if [[ -e data ]]; then echo "data already exists"; exit 1; fi
mkdir data
for f in BDS BLD CLT EPR FML GNM HST LTT PNM RLW ROD SAT SCL SNT TTL; do
    if [[ ! -e data/$f ]]; then 
        echo "Extracting $f";
        mkdir data/$f
        for g in orig/$f/*; do
            g1=`basename $g .xml`;
            echo "cat $g | script/extract-sentences.pl data/$f/$g1"
            cat $g | script/extract-sentences.pl data/$f/$g1
            if [[ $g1 = "${f}00004" ]]; then
                g2="test"
            elif [[ $g1 = "${f}00002" ]]; then
                g2="dev"
            elif [[ $g1 = "${f}00003" ]]; then
                g2="tune"
            else
                g2="train"
            fi
            cat data/$f/$g1.en >> data/$f/$f-$g2.en
            cat data/$f/$g1.ja >> data/$f/$f-$g2.ja
        done
    fi
done
if [[ -e data/orig ]]; then echo "data/orig already exists"; exit 1; fi
mkdir data/orig
for f in tune dev test train; do
    if [[ ! -e data/orig/kyoto-$f.en ]]; then
        echo "Making $f Set"
        cat data/*/*-$f.en >> data/orig/kyoto-$f.en
        cat data/*/*-$f.ja >> data/orig/kyoto-$f.ja
    fi
done

# 3) tokenize, clean, and lowercase the corpus
echo "Tokenizing"
if [[ -e data/tok ]]; then echo "data/tok already exists"; exit 1; fi
for f in tune dev test train; do
    echo " $f"
    $MT_SCRIPTS/tokenizer.perl en < data/orig/kyoto-$f.en > data/tok/kyoto-$f.en
    $KYTEA -model script/kytea-kyoto.mod < data/orig/kyoto-$f.ja > data/tok/kyoto-$f.ja
done
echo "Cleaning"
$MOSES_DIR/scripts/training/clean-corpus-n.perl data/tok/kyoto-train ja en data/tok/kyoto-train.cln 1 40
echo "Lowercasing"
if [[ -e data/low ]]; then echo "data/low already exists"; exit 1; fi
for f in data/tok/*; do
    echo " $f"
    $MT_SCRIPTS/lowercase.perl < $f > data/low/`basename $f`;
done

# 4) train the language models
if [[ -e lm ]]; then echo "lm already exists"; exit 1; fi
for f in en ja; do
    echo "Training $f LM"
    $NGRAM_COUNT -order 5 -interpolate -kndiscount -text data/low/kyoto-train.cln.$f -lm lm/kyoto-train.$f
done

for f in ja en; do
    # Set up variables
    if [[ $f = "ja" ]]; then
        e="en"; 
        attach="left"
    else 
        e="ja";
        attach="right"
    fi

    # 5) train alignments with GIZA++
    ALIGN_DIR="$WORK_DIR/$f-$e-align"
    if [[ -e $ALIGN_DIR ]]; then echo "$ALIGN_DIR already exists"; exit 1; fi
    echo "Training GIZA++ alignments for $f-$e"
$MOSES_DIR/scripts/training/train-model.perl -parallel -cores $THREADS -external-bin-dir $EXTERNAL_BIN_DIR -last-step 3 -root-dir $ALIGN_DIR -corpus data/low/kyoto-train.cln -f $f -e $e -alignment grow-diag-final-and -reordering msd-bidirectional-fe -lm 0:5:$WD/lm/kyoto-train.$e:0

    # 6) train and run the lader reorderer
    REORDER_DIR="$WORK_DIR/$f-$e-reorder"
    if [[ -e $REORDER_DIR ]]; then echo "$REORDER_DIR already exists"; exit 1; fi
    mkdir $REORDER_DIR $REORDER_DIR/train $REORDER_DIR/reordered
    # Build the phrase table
    [[ -e $REORDER_DIR/train/kyoto-train.cln.pt ]] || $LADER_DIR/script/contiguous-extract.pl data/low/kyoto-train.cln.$f data/low/kyoto-train.cln.$e $ALIGN_DIR/model/aligned.grow-diag-final-and > $REORDER_DIR/train/kyoto-train.cln.pt
    # Annotate the reordering training corpora with classes
    $LADER_DIR/script/add-classes.pl $ALIGN_DIR/corpus/$f.vcb.classes < orig/alignments/$f-$e.$f > $REORDER_DIR/train/$f-$e.$f.cls
    paste orig/alignments/$f-$e.$f $REORDER_DIR/train/$f-$e.$f.cls > $REORDER_DIR/train/$f-$e.$f.annot
    # Train lader
    $TRAIN_LADER -cost 1e-3 -attach_null left -feature_profile "seq=dict=$WD/$REORDER_DIR/train/kyoto-train.cln.pt,LL%SL%ET,RR%SR%ET,LR%LR%ET,RL%RL%ET,O%SL%SR%ET,I%LR%RL%ET,Q%SQE0%ET,Q0%SQ#00%ET,Q1%SQ#01%ET,Q2%SQ#02%ET,CL%CL%ET,B%SB%ET,A%SA%ET,N%SN%ET,BIAS%ET|seq=LL%SL%ET,RR%SR%ET,LR%LR%ET,RL%RL%ET,B%SB%ET,A%SA%ET,O%SL%SR%ET,I%LR%RL%ET" -iterations 500 -model_out $REORDER_DIR/train/$f-$e.mod -source_in $REORDER_DIR/train/$f-$e.$f.annot -align_in orig/alignments/$f-$e.align
    # Annotate each translation training corpus
    for t in dev tune test train.cln; do
        echo "$LADER_DIR/script/add-classes.pl $ALIGN_DIR/corpus/$f.vcb.classes < data/low/kyoto-$t.$f > $REORDER_DIR/reordered/kyoto-$t.$f.cls"
        $LADER_DIR/script/add-classes.pl $ALIGN_DIR/corpus/$f.vcb.classes < data/low/kyoto-$t.$f > $REORDER_DIR/reordered/kyoto-$t.$f.cls
        echo "paste data/low/kyoto-$t.$f $REORDER_DIR/reordered/kyoto-$t.$f.cls > $REORDER_DIR/reordered/kyoto-$t.$f.annot"
        paste data/low/kyoto-$t.$f $REORDER_DIR/reordered/kyoto-$t.$f.cls > $REORDER_DIR/reordered/kyoto-$t.$f.annot
        echo "$LADER -threads $THREADS -model $REORDER_DIR/train/$f-$e.mod < $REORDER_DIR/reordered/kyoto-$t.$f.annot > $REORDER_DIR/reordered/kyoto-$t.$f 2> $REORDER_DIR/$REORDER_DIR.log"
        $LADER -threads $THREADS -model $REORDER_DIR/train/$f-$e.mod < $REORDER_DIR/reordered/kyoto-$t.$f.annot > $REORDER_DIR/reordered/kyoto-$t.$f 2> $REORDER_DIR/lader.log
        ln -s $WD/data/low/kyoto-$t.$e $REORDER_DIR/reordered/kyoto-$t.$e
    done

    # 6) build the model
    MODEL_DIR="$WORK_DIR/$f-$e"
    if [[ -e $MODEL_DIR ]]; then echo "$MODEL_DIR already exists"; exit 1; fi
    echo "Training Moses for $f-$e"
    $MOSES_DIR/scripts/training/train-model.perl -parallel -cores $THREADS -external-bin-dir $EXTERNAL_BIN_DIR -root-dir $MODEL_DIR -corpus $REORDER_DIR/reordered/kyoto-train.cln -f $f -e $e -alignment grow-diag-final-and -reordering msd-bidirectional-fe -lm 0:5:$WD/lm/kyoto-train.$e:0

    # 7) do mert
    echo "Doing MERT for $f-$e"
    $MOSES_DIR/scripts/training/mert-moses.pl $WD/$REORDER_DIR/reordered/kyoto-dev.$f $WD/$REORDER_DIR/reordered/kyoto-dev.$e $MOSES $WD/$MODEL_DIR/model/moses.ini --mertdir $MOSES_DIR/bin --working-dir $WD/$MODEL_DIR/tuning --decoder-flags "-threads $THREADS -v 0"

    # 8) test
    [[ -e $MODEL_DIR/evaluation ]] || mkdir $MODEL_DIR/evaluation
    for t in dev test; do
        echo "Testing and evaluating for $f-$e, $t"
        # filter
        echo "$MOSES_DIR/scripts/training/filter-model-given-input.pl $MODEL_DIR/evaluation/filtered.$t $MODEL_DIR/tuning/moses.ini $REORDER_DIR/reordered/kyoto-$t.$f"
        $MOSES_DIR/scripts/training/filter-model-given-input.pl $MODEL_DIR/evaluation/filtered.$t $MODEL_DIR/tuning/moses.ini $REORDER_DIR/reordered/kyoto-$t.$f
        # translate
        echo "$MOSES -config $MODEL_DIR/evaluation/filtered.$t/moses.ini < $REORDER_DIR/reordered/kyoto-$t.$f 1> $MODEL_DIR/evaluation/$t.out 2> $MODEL_DIR/evaluation/$t.err"
        $MOSES -config $MODEL_DIR/evaluation/filtered.$t/moses.ini < $REORDER_DIR/reordered/kyoto-$t.$f 1> $MODEL_DIR/evaluation/$t.out 2> $MODEL_DIR/evaluation/$t.err
        # note: before evaluation of Japanese files, it may be desirable to re-tokenize the file
        #  using KyTea to match the reference more closely if you are using a different
        #  segmentation standard
        echo "$MOSES_DIR/scripts/generic/multi-bleu.perl $REORDER_DIR/reordered/kyoto-$t.$e < $MODEL_DIR/evaluation/$t.out > $MODEL_DIR/evaluation/$t.grade"
        $MOSES_DIR/scripts/generic/multi-bleu.perl $REORDER_DIR/reordered/kyoto-$t.$e < $MODEL_DIR/evaluation/$t.out > $MODEL_DIR/evaluation/$t.grade
    done

done

grep BLEU {ja-en,en-ja}/evaluation/*.grade
